@@ -14,6 +14,9 @@ namespace Webmozart\Assert;
 use BadMethodCallException;
 use InvalidArgumentException;
 use Traversable;
+use Exception;
+use Throwable;
+use Closure;
 
 /**
  * Efficient assertions to validate the input/output of your methods.
@@ -26,6 +29,7 @@ use Traversable;
  * @method static void nullOrNumeric($value, $message = '')
  * @method static void nullOrBoolean($value, $message = '')
  * @method static void nullOrScalar($value, $message = '')
+ * @method static void nullOrObject($value, $message = '')
  * @method static void nullOrResource($value, $type = null, $message = '')
  * @method static void nullOrIsCallable($value, $message = '')
  * @method static void nullOrIsArray($value, $message = '')
@@ -68,6 +72,14 @@ use Traversable;
  * @method static void nullOrClassExists($value, $message = '')
  * @method static void nullOrSubclassOf($value, $class, $message = '')
  * @method static void nullOrImplementsInterface($value, $interface, $message = '')
+ * @method static void nullOrPropertyExists($value, $property, $message = '')
+ * @method static void nullOrPropertyNotExists($value, $property, $message = '')
+ * @method static void nullOrMethodExists($value, $method, $message = '')
+ * @method static void nullOrMethodNotExists($value, $method, $message = '')
+ * @method static void nullOrKeyExists($value, $key, $message = '')
+ * @method static void nullOrKeyNotExists($value, $key, $message = '')
+ * @method static void nullOrCount($value, $key, $message = '')
+ * @method static void nullOrUuid($values, $message = '')
  * @method static void allString($values, $message = '')
  * @method static void allStringNotEmpty($values, $message = '')
  * @method static void allInteger($values, $message = '')
@@ -76,6 +88,7 @@ use Traversable;
  * @method static void allNumeric($values, $message = '')
  * @method static void allBoolean($values, $message = '')
  * @method static void allScalar($values, $message = '')
+ * @method static void allObject($values, $message = '')
  * @method static void allResource($values, $type = null, $message = '')
  * @method static void allIsCallable($values, $message = '')
  * @method static void allIsArray($values, $message = '')
@@ -120,6 +133,14 @@ use Traversable;
  * @method static void allClassExists($values, $message = '')
  * @method static void allSubclassOf($values, $class, $message = '')
  * @method static void allImplementsInterface($values, $interface, $message = '')
+ * @method static void allPropertyExists($values, $property, $message = '')
+ * @method static void allPropertyNotExists($values, $property, $message = '')
+ * @method static void allMethodExists($values, $method, $message = '')
+ * @method static void allMethodNotExists($values, $method, $message = '')
+ * @method static void allKeyExists($values, $key, $message = '')
+ * @method static void allKeyNotExists($values, $key, $message = '')
+ * @method static void allCount($values, $key, $message = '')
+ * @method static void allUuid($values, $message = '')
  *
  * @since  1.0
  *
@@ -130,25 +151,25 @@ class Assert
     public static function string($value, $message = '')
     {
         if (!is_string($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a string. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
 
     public static function stringNotEmpty($value, $message = '')
     {
-        self::string($value, $message);
-        self::notEmpty($value, $message);
+        static::string($value, $message);
+        static::notEmpty($value, $message);
     }
 
     public static function integer($value, $message = '')
     {
         if (!is_int($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an integer. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -156,9 +177,9 @@ class Assert
     public static function integerish($value, $message = '')
     {
         if (!is_numeric($value) || $value != (int) $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an integerish value. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -166,9 +187,9 @@ class Assert
     public static function float($value, $message = '')
     {
         if (!is_float($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a float. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -176,9 +197,9 @@ class Assert
     public static function numeric($value, $message = '')
     {
         if (!is_numeric($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a numeric. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -186,9 +207,9 @@ class Assert
     public static function boolean($value, $message = '')
     {
         if (!is_bool($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a boolean. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -196,9 +217,19 @@ class Assert
     public static function scalar($value, $message = '')
     {
         if (!is_scalar($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a scalar. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
+            ));
+        }
+    }
+
+    public static function object($value, $message = '')
+    {
+        if (!is_object($value)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Expected an object. Got: %s',
+                static::typeToString($value)
             ));
         }
     }
@@ -206,16 +237,16 @@ class Assert
     public static function resource($value, $type = null, $message = '')
     {
         if (!is_resource($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a resource. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
 
         if ($type && $type !== get_resource_type($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a resource of type %2$s. Got: %s',
-                self::typeToString($value),
+                static::typeToString($value),
                 $type
             ));
         }
@@ -224,9 +255,9 @@ class Assert
     public static function isCallable($value, $message = '')
     {
         if (!is_callable($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a callable. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -234,9 +265,9 @@ class Assert
     public static function isArray($value, $message = '')
     {
         if (!is_array($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an array. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -244,9 +275,9 @@ class Assert
     public static function isTraversable($value, $message = '')
     {
         if (!is_array($value) && !($value instanceof Traversable)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a traversable. Got: %s',
-                self::typeToString($value)
+                static::typeToString($value)
             ));
         }
     }
@@ -254,9 +285,9 @@ class Assert
     public static function isInstanceOf($value, $class, $message = '')
     {
         if (!($value instanceof $class)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an instance of %2$s. Got: %s',
-                self::typeToString($value),
+                static::typeToString($value),
                 $class
             ));
         }
@@ -265,9 +296,9 @@ class Assert
     public static function notInstanceOf($value, $class, $message = '')
     {
         if ($value instanceof $class) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an instance other than %2$s. Got: %s',
-                self::typeToString($value),
+                static::typeToString($value),
                 $class
             ));
         }
@@ -276,9 +307,9 @@ class Assert
     public static function isEmpty($value, $message = '')
     {
         if (!empty($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an empty value. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -286,9 +317,9 @@ class Assert
     public static function notEmpty($value, $message = '')
     {
         if (empty($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a non-empty value. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -296,9 +327,9 @@ class Assert
     public static function null($value, $message = '')
     {
         if (null !== $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected null. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -306,7 +337,7 @@ class Assert
     public static function notNull($value, $message = '')
     {
         if (null === $value) {
-            throw new InvalidArgumentException(
+            static::reportInvalidArgument(
                 $message ?: 'Expected a value other than null.'
             );
         }
@@ -315,9 +346,9 @@ class Assert
     public static function true($value, $message = '')
     {
         if (true !== $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to be true. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -325,9 +356,9 @@ class Assert
     public static function false($value, $message = '')
     {
         if (false !== $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to be false. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -335,10 +366,10 @@ class Assert
     public static function eq($value, $value2, $message = '')
     {
         if ($value2 != $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value equal to %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($value2)
+                static::valueToString($value),
+                static::valueToString($value2)
             ));
         }
     }
@@ -346,9 +377,9 @@ class Assert
     public static function notEq($value, $value2, $message = '')
     {
         if ($value2 == $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a different value than %s.',
-                self::valueToString($value2)
+                static::valueToString($value2)
             ));
         }
     }
@@ -356,10 +387,10 @@ class Assert
     public static function same($value, $value2, $message = '')
     {
         if ($value2 !== $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value identical to %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($value2)
+                static::valueToString($value),
+                static::valueToString($value2)
             ));
         }
     }
@@ -367,9 +398,9 @@ class Assert
     public static function notSame($value, $value2, $message = '')
     {
         if ($value2 === $value) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value not identical to %s.',
-                self::valueToString($value2)
+                static::valueToString($value2)
             ));
         }
     }
@@ -377,10 +408,10 @@ class Assert
     public static function greaterThan($value, $limit, $message = '')
     {
         if ($value <= $limit) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value greater than %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($limit)
+                static::valueToString($value),
+                static::valueToString($limit)
             ));
         }
     }
@@ -388,10 +419,10 @@ class Assert
     public static function greaterThanEq($value, $limit, $message = '')
     {
         if ($value < $limit) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value greater than or equal to %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($limit)
+                static::valueToString($value),
+                static::valueToString($limit)
             ));
         }
     }
@@ -399,10 +430,10 @@ class Assert
     public static function lessThan($value, $limit, $message = '')
     {
         if ($value >= $limit) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value less than %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($limit)
+                static::valueToString($value),
+                static::valueToString($limit)
             ));
         }
     }
@@ -410,10 +441,10 @@ class Assert
     public static function lessThanEq($value, $limit, $message = '')
     {
         if ($value > $limit) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value less than or equal to %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($limit)
+                static::valueToString($value),
+                static::valueToString($limit)
             ));
         }
     }
@@ -421,11 +452,11 @@ class Assert
     public static function range($value, $min, $max, $message = '')
     {
         if ($value < $min || $value > $max) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value between %2$s and %3$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($min),
-                self::valueToString($max)
+                static::valueToString($value),
+                static::valueToString($min),
+                static::valueToString($max)
             ));
         }
     }
@@ -433,10 +464,10 @@ class Assert
     public static function oneOf($value, array $values, $message = '')
     {
         if (!in_array($value, $values, true)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected one of: %2$s. Got: %s',
-                self::valueToString($value),
-                implode(', ', array_map(array(__CLASS__, 'valueToString'), $values))
+                static::valueToString($value),
+                implode(', ', array_map(array('static', 'valueToString'), $values))
             ));
         }
     }
@@ -444,10 +475,10 @@ class Assert
     public static function contains($value, $subString, $message = '')
     {
         if (false === strpos($value, $subString)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($subString)
+                static::valueToString($value),
+                static::valueToString($subString)
             ));
         }
     }
@@ -455,10 +486,10 @@ class Assert
     public static function startsWith($value, $prefix, $message = '')
     {
         if (0 !== strpos($value, $prefix)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to start with %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($prefix)
+                static::valueToString($value),
+                static::valueToString($prefix)
             ));
         }
     }
@@ -475,20 +506,20 @@ class Assert
         }
 
         if (!$valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to start with a letter. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
 
     public static function endsWith($value, $suffix, $message = '')
     {
-        if ($suffix !== substr($value, -self::strlen($suffix))) {
-            throw new InvalidArgumentException(sprintf(
+        if ($suffix !== substr($value, -static::strlen($suffix))) {
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to end with %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($suffix)
+                static::valueToString($value),
+                static::valueToString($suffix)
             ));
         }
     }
@@ -496,9 +527,9 @@ class Assert
     public static function regex($value, $pattern, $message = '')
     {
         if (!preg_match($pattern, $value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The value %s does not match the expected pattern.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -511,9 +542,9 @@ class Assert
         setlocale(LC_CTYPE, $locale);
 
         if ($valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain only letters. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -526,9 +557,9 @@ class Assert
         setlocale(LC_CTYPE, $locale);
 
         if ($valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain digits only. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -541,9 +572,9 @@ class Assert
         setlocale(LC_CTYPE, $locale);
 
         if ($valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain letters and digits only. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -556,9 +587,9 @@ class Assert
         setlocale(LC_CTYPE, $locale);
 
         if ($valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain lowercase characters only. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -571,19 +602,19 @@ class Assert
         setlocale(LC_CTYPE, $locale);
 
         if ($valid) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain uppercase characters only. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
 
     public static function length($value, $length, $message = '')
     {
-        if ($length !== self::strlen($value)) {
-            throw new InvalidArgumentException(sprintf(
+        if ($length !== static::strlen($value)) {
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain %2$s characters. Got: %s',
-                self::valueToString($value),
+                static::valueToString($value),
                 $length
             ));
         }
@@ -591,10 +622,10 @@ class Assert
 
     public static function minLength($value, $min, $message = '')
     {
-        if (self::strlen($value) < $min) {
-            throw new InvalidArgumentException(sprintf(
+        if (static::strlen($value) < $min) {
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain at least %2$s characters. Got: %s',
-                self::valueToString($value),
+                static::valueToString($value),
                 $min
             ));
         }
@@ -602,10 +633,10 @@ class Assert
 
     public static function maxLength($value, $max, $message = '')
     {
-        if (self::strlen($value) > $max) {
-            throw new InvalidArgumentException(sprintf(
+        if (static::strlen($value) > $max) {
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain at most %2$s characters. Got: %s',
-                self::valueToString($value),
+                static::valueToString($value),
                 $max
             ));
         }
@@ -613,12 +644,12 @@ class Assert
 
     public static function lengthBetween($value, $min, $max, $message = '')
     {
-        $length = self::strlen($value);
+        $length = static::strlen($value);
 
         if ($length < $min || $length > $max) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a value to contain between %2$s and %3$s characters. Got: %s',
-                self::valueToString($value),
+                static::valueToString($value),
                 $min,
                 $max
             ));
@@ -627,36 +658,36 @@ class Assert
 
     public static function fileExists($value, $message = '')
     {
-        self::string($value);
+        static::string($value);
 
         if (!file_exists($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The file %s does not exist.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
 
     public static function file($value, $message = '')
     {
-        self::fileExists($value, $message);
+        static::fileExists($value, $message);
 
         if (!is_file($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The path %s is not a file.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
 
     public static function directory($value, $message = '')
     {
-        self::fileExists($value, $message);
+        static::fileExists($value, $message);
 
         if (!is_dir($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The path %s is no directory.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -664,9 +695,9 @@ class Assert
     public static function readable($value, $message = '')
     {
         if (!is_readable($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The path %s is not readable.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -674,9 +705,9 @@ class Assert
     public static function writable($value, $message = '')
     {
         if (!is_writable($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'The path %s is not writable.',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -684,9 +715,9 @@ class Assert
     public static function classExists($value, $message = '')
     {
         if (!class_exists($value)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an existing class name. Got: %s',
-                self::valueToString($value)
+                static::valueToString($value)
             ));
         }
     }
@@ -694,10 +725,10 @@ class Assert
     public static function subclassOf($value, $class, $message = '')
     {
         if (!is_subclass_of($value, $class)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected a sub-class of %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($class)
+                static::valueToString($value),
+                static::valueToString($class)
             ));
         }
     }
@@ -705,10 +736,50 @@ class Assert
     public static function implementsInterface($value, $interface, $message = '')
     {
         if (!in_array($interface, class_implements($value))) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected an implementation of %2$s. Got: %s',
-                self::valueToString($value),
-                self::valueToString($interface)
+                static::valueToString($value),
+                static::valueToString($interface)
+            ));
+        }
+    }
+
+    public static function propertyExists($classOrObject, $property, $message = '')
+    {
+        if (!property_exists($classOrObject, $property)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Expected the property %s to exist.',
+                static::valueToString($property)
+            ));
+        }
+    }
+
+    public static function propertyNotExists($classOrObject, $property, $message = '')
+    {
+        if (property_exists($classOrObject, $property)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Expected the property %s to not exist.',
+                static::valueToString($property)
+            ));
+        }
+    }
+
+    public static function methodExists($classOrObject, $method, $message = '')
+    {
+        if (!method_exists($classOrObject, $method)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Expected the method %s to exist.',
+                static::valueToString($method)
+            ));
+        }
+    }
+
+    public static function methodNotExists($classOrObject, $method, $message = '')
+    {
+        if (method_exists($classOrObject, $method)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Expected the method %s to not exist.',
+                static::valueToString($method)
             ));
         }
     }
@@ -716,9 +787,9 @@ class Assert
     public static function keyExists($array, $key, $message = '')
     {
         if (!array_key_exists($key, $array)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected the key %s to exist.',
-                self::valueToString($key)
+                static::valueToString($key)
             ));
         }
     }
@@ -726,13 +797,66 @@ class Assert
     public static function keyNotExists($array, $key, $message = '')
     {
         if (array_key_exists($key, $array)) {
-            throw new InvalidArgumentException(sprintf(
+            static::reportInvalidArgument(sprintf(
                 $message ?: 'Expected the key %s to not exist.',
-                self::valueToString($key)
+                static::valueToString($key)
             ));
         }
     }
 
+    public static function count($array, $number, $message = '')
+    {
+        static::eq(
+            count($array),
+            $number,
+            $message ?: sprintf('Expected an array to contain %d elements. Got: %d.', $number, count($array))
+        );
+    }
+
+    public static function uuid($value, $message = '')
+    {
+        $value = str_replace(array('urn:', 'uuid:', '{', '}'), '', $value);
+
+        // The nil UUID is special form of UUID that is specified to have all
+        // 128 bits set to zero.
+        if ('00000000-0000-0000-0000-000000000000' === $value) {
+            return;
+        }
+
+        if (!preg_match('/^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}$/', $value)) {
+            static::reportInvalidArgument(sprintf(
+                $message ?: 'Value %s is not a valid UUID.',
+                static::valueToString($value)
+            ));
+        }
+    }
+
+    public static function throws(Closure $expression, $class = 'Exception', $message = '')
+    {
+        static::string($class);
+
+        $actual = 'none';
+        try {
+            $expression();
+        } catch (Exception $e) {
+            $actual = get_class($e);
+            if ($e instanceof $class) {
+                return;
+            }
+        } catch (Throwable $e) {
+            $actual = get_class($e);
+            if ($e instanceof $class) {
+                return;
+            }
+        }
+
+        static::reportInvalidArgument($message ?: sprintf(
+            'Expected to throw "%s", got "%s"',
+            $class,
+            $actual
+        ));
+    }
+
     public static function __callStatic($name, $arguments)
     {
         if ('nullOr' === substr($name, 0, 6)) {
@@ -745,7 +869,7 @@ class Assert
         }
 
         if ('all' === substr($name, 0, 3)) {
-            self::isTraversable($arguments[0]);
+            static::isTraversable($arguments[0]);
 
             $method = lcfirst(substr($name, 3));
             $args = $arguments;
@@ -813,6 +937,11 @@ class Assert
         return mb_strwidth($value, $encoding);
     }
 
+    protected static function reportInvalidArgument($message)
+    {
+        throw new InvalidArgumentException($message);
+    }
+
     private function __construct()
     {
     }
